/*
 * Copyright 2017 aquenos GmbH.
 * All rights reserved.
 * 
 * Please refer to the file NOTICE.txt in the root of the source distribution or
 * in the META-INF directory of the binary archive for licensing information.
 */

package com.aquenos.epics.jackie.diirt.vtype;

import java.nio.ShortBuffer;
import java.time.Instant;

import org.diirt.util.array.ListShort;
import org.diirt.vtype.VShortArray;
import org.diirt.vtype.VTypeToString;

import com.aquenos.epics.jackie.common.value.ChannelAccessControlsShort;
import com.aquenos.epics.jackie.common.value.ChannelAccessTimeShort;

/**
 * {@link VShortArray} implementation that is backed by a
 * {@link ChannelAccessControlsShort} and {@link ChannelAccessTimeShort}.
 * 
 * @author Sebastian Marsching
 */
public final class JackieVShortArray extends
        JackieVNumberArray<ChannelAccessControlsShort, ChannelAccessTimeShort>
        implements VShortArray {

    /**
     * <p>
     * Creates a value backed by the specified {@code controlsValue} and
     * {@code timeValue}. The {@code controlsValue} is used for the meta-data
     * (engineering units, limits, etc.) and the {@code timeValue} is used for
     * the alarm status, time-stamp, and value.
     * </p>
     * 
     * <p>
     * If {@code disconnectTime} is not <code>null</code>, it is used as the
     * time-stamp and the alarm severity is set to {@code UNDEFINED}.
     * </p>
     * 
     * @param controlsValue
     *            value used for meta-data (engineering units, limits, etc.).
     * @param timeValue
     *            value used for alarm status, time-stamp, and value.
     * @param disconnectTime
     *            time at which the channel was disconnected. Usually this
     *            should be <code>null</code>. If it is not <code>null</code>,
     *            the alarm severity is set to {@code UNDEFINED} and this
     *            parameter is used as the time-stamp (instead of using the
     *            time-stamp from the {@code timeValue}.
     */
    public JackieVShortArray(ChannelAccessControlsShort controlsValue,
            ChannelAccessTimeShort timeValue, Instant disconnectTime) {
        super(controlsValue, timeValue, disconnectTime, true);
    }

    @Override
    public ListShort getData() {
        ShortBuffer buffer = timeValue.getValue();
        return new ListShort() {
            @Override
            public int size() {
                return buffer.remaining();
            }

            @Override
            public short getShort(int index) {
                return buffer.get(index);
            }
        };
    }

    @Override
    public String toString() {
        return VTypeToString.toString(this);
    }

}
